/* $Header: /usr2/brugere/jbln/uc133/src/RCS/input.c,v 1.1 1995/02/17 22:06:47 jbln Exp $ */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>


#include "keys.h"
#include "uc.h"
#include "input.h"




#ifdef EXAMPLE
main()
{
   INP *i;
   char str[21];
   int c;

   strcpy(str, "12345");
   if ((i=inp_new(2,2,str,6,10,INPKEEP)) == NULL)
      exit(1);

   do
   {
      c = getch();
      if (c == -59)
	 inp_addstr(i, "+-+");
   }
   while (inp_addchr(i,c) == INPMORE);

   printf("\n\n\rSTR = %s\n\r", str);
}
#endif


/*******************************************************************
   Create input
   ------------
   Input:
      x,y = screenpos.
      str = input string
      inpsize = number of chars to show
      strsize = max. number of chars to insert in string.
      flags = INPNUM | INPHEX | INPHIDE | INPKEEP

   Returns
      NULL on error or a new inputpointer;
*******************************************************************/
INP *inp_new(x,y,str,inpsize,strsize,flags)
int x,y;
char *str;
int inpsize, strsize;
int flags;
{
   INP *i;

   if ( (i=(INP *)malloc(sizeof(INP))) == (INP *)NULL )
      return(NULL);
   if ( (i->savestr=(char *)malloc(strlen(str)+1)) == (char *)NULL )
   {
      free(i);
      return(NULL);
   }

   i->x = x;
   i->y = y;
   i->str = str;
   i->inpsize = inpsize;
   i->strsize = strsize;
   i->flags = flags;
   i->first = 1;
   i->p = i->o = i->len = 0;

   if ((flags & INPKEEP) == INPKEEP)
   {
      strcpy(i->savestr, str);
      if ((i->len=strlen(str)) >= inpsize )
      {
	 i->p = inpsize-1;
	 i->o = i->len - i->p;
      }
      else
      {
	 i->p = i->len;
	 i->o = 0;
      }
      attron(A_REVERSE);
   }
   else
   {
      i->str[0] = 0;
   }

   inp_putstr(i, &str[i->o]);
   move(y,x+i->p);

   if ((flags & INPKEEP) == INPKEEP)
      attroff(A_REVERSE);
   refresh();

   return(i);
}


/*******************************************************************
   Delete inputpointer
   -------------------
   Input:
      i = Valid inputpointer, made with inp_new.

   Returns
      Always 1
*******************************************************************/
inp_del(i)
INP *i;
{
   free(i->savestr);
   free(i);

   return(1);
}


/*******************************************************************
   Add character to inputstring
   ----------------------------
   Input:
      i = Valid inputpointer, created with inp_new.
      c = New character to insert.

   Returns:
      INPMORE | INPESC | INPENTER
*******************************************************************/
inp_addchr(i, c)
INP *i;
int c;
{
   int at_end=0;
   int showmode=0;
   int old_p;
   int retcode=INPMORE;
   int n;

   if (i == (INP *)NULL)
      return(0);

   if (i->first  &&  (i->flags & INPKEEP) == INPKEEP)
      inp_test_first(i,c);

   old_p = i->p;
   if (i->str[i->o+i->p] == 0)
      at_end = 1;

   if (inp_isinput(i,c)  &&  i->len <= i->strsize-1)
   {
	 /* Move all symbols right of cur. pos */
      if (!at_end)
      {
	 i->str[i->len+1] = 0;
	 for (n=i->len ;  n > i->o+i->p  ;  n--)
	    i->str[n] = i->str[n-1];
      }
      else
      if (i->p < i->inpsize-1)
	 showmode = 1;

      i->str[i->o + i->p] = c;
      i->p++;
      if (at_end)
	 i->str[i->o + i->p] = 0;

      if (i->p >= i->inpsize)
      {
	 i->o++;
	 i->p = i->inpsize - 1;
      }

      i->len++;
   }
   else
   if (inp_isbackspace(c)  &&  i->p+i->o > 0)
   {
      if (i->o > 0)
	 i->o--;
      else
      {
	 if (at_end)
	    showmode = 3;
	 i->p--;
      }

      for (n=i->o+i->p ; i->str[n]!=0 ; n++)
	 i->str[n] = i->str[n+1];
      i->str[n] = 0;
      i->len--;
   }
   else
   if (inp_isleft(c))
   {
      i->p--;
      if (i->p < 0)
      {
	 i->p = 0;
	 if (i->o > 0)
	    i->o--;
	 else
	    showmode = 2;
      }
      else
	 showmode = 2;
   }
   else
   if (inp_isright(c)  &&  i->str[i->p+i->o]!=0)
   {
      i->p++;
      if (i->p >= i->inpsize)
      {
	 i->p--;
	 i->o++;
      }
      else
	 showmode = 2;
   }
   else
   if (inp_isenter(c))
   {
      retcode = INPENTER;
   }
   else
   if (inp_isescape(c))
   {
      retcode = INPESC;
      if ((i->flags & INPKEEP) == INPKEEP)
      {
	 strcpy(i->str, i->savestr);
	 i->len = strlen(i->str);
	 if (i->len >= i->inpsize)
	 {
	    i->p = i->inpsize - 1;
	    i->o = i->len - i->p;
	 }
	 else
	 {
	    i->p = i->len;
	    i->o = 0;
	 }
      }
      else
      {
	 i->str[0] = 0;
	 i->o = i->p = i->len = 0;
      }
      inp_putstr(i, i->str);
   }
   else
   if (inp_isbackspace(c)  &&  i->p+i->o == 0)
      showmode = 1;
   else
      showmode = 2;

   if (showmode == 0)
   {
      for (n=0 ; n<i->inpsize & i->str[n+i->o]!=0 ; n++)
	 if ((i->flags & INPHIDE) == INPHIDE)
	    mvaddch(i->y, i->x+n, '*');
	 else
	    mvaddch(i->y, i->x+n, i->str[n+i->o]);

      if (n < i->inpsize)
	 mvaddch(i->y, i->x+n, ' ');
   }
   else
   {
      if (showmode == 1)
      {
	 if ((i->flags & INPHIDE) == INPHIDE)
	    mvaddch(i->y, i->x + old_p, '*');
	 else
	    mvaddch(i->y, i->x + old_p, c);
      }
      else
      if (showmode == 3)
	 mvaddch(i->y, i->x + i->p, ' ');
   }

   move(i->y, i->x + i->p);
   refresh();

   i->first = 0;
   return(retcode);
}


/*******************************************************************
   Add string to inputstring
   ----------------------------
   Input:
      i = Valid inputpointer, created with inp_new.
      c = New string to add to inputstring

   Returns:
      0 if inputstr is full.
      1 otherwise.

   Comments:
      No tests are done to ensure INPHEX | INPNUM | INPHIDE !
*******************************************************************/
inp_addstr(i, str)
INP *i;
char *str;
{
   int n,l;

   if (i->first  &&  (i->flags & INPKEEP) == INPKEEP)
   {
      i->p = i->o = i->len = 0;
      i->str[0] = 0;
      inp_clear_scr(i);
   }

   l = strlen(str);
   if (strlen(i->str) + l > i->strsize)
      return(0);

   i->str[i->len+l] = 0;
   for (n=i->len-1 ; n >= i->o+i->p ; n--)
      i->str[n+l] = i->str[n];

   for (n=0 ; n<l ; n++)
      i->str[i->o+i->p+n] = str[n];

   i->len += l;

   if (i->p + l >= i->inpsize)
   {
      i->p = i->inpsize-1;
      i->o = i->len - i->p;
   }
   else
      i->p += l;

   inp_putstr(i,&i->str[i->o]);
   refresh();

   return(1);
}


inp_update(i)
INP *i;
{
   inp_putstr(i, &i->str[i->o]);
   move(i->y, i->x+i->p);
}


inp_clear(i)
INP *i;
{
   i->o = i->p = i->len = 0;
   i->str[0] = 0;
   inp_clear_scr(i);
}


inp_repos(i, x,y, inpsize)
INP *i;
int x,y;
int inpsize;
{
   inp_clear_scr(i);
   i->x = x;
   i->y = y;
   i->inpsize = inpsize;
   inp_update(i);
}


/*******************************************************************
   Normal input routine
   --------------------
   Input:
      x,y = screenpos.
      str = input string
      inpsize = number of chars to show
      strsize = max. number of chars to insert in string.
      flags = INPNUM | INPHEX | INPHIDE | INPKEEP

   Returns:
      INPENTER | INPESC
*******************************************************************/
inp_input(x,y,str,inpsize,strsize,flags)
int x,y;
char *str;
int inpsize, strsize;
int flags;
{
   INP *i;
   int retcode, c;

   if ((i=inp_new(x,y,str,inpsize,strsize,flags)) == NULL)
      return(0);

   do
   {
      c = ti_getch();
   }
   while ((retcode=inp_addchr(i,c)) == INPMORE);

   inp_del(i);
   return(retcode);
}




/*******************************************************************
   INTERNAL FUNCTIONS ONLY
*******************************************************************/

static inp_isinput(i,c)
INP *i;
int c;
{
   if (inp_isbackspace(c) || inp_isleft(c) || inp_isright(c)
       || inp_isenter(c)  ||  inp_isescape(c))
      return(0);

   if ((i->flags & INPNUM) == INPNUM)
      return( c>='0'  &&  c<='9' );

   if ((i->flags & INPHEX) == INPHEX)
      return( c>='0' && c<='9'  ||  c>='a' && c<='f'  ||  c>='A' && c<='F' );
   return( c>=32  &&  c<=127 );
}


static inp_test_first(i,c)
INP *i;
int c;
{
   if (inp_isinput(i,c))
   {
      i->p = i->o = i->len = 0;
      i->str[0] = 0;
      inp_clear_scr(i);
   }
   else
      inp_putstr(i, i->str);

   move(i->y,i->x+i->p);
}


static inp_putstr(i, p)
INP *i;
char *p;
{
   int n;

   for (n=0 ; p[n]!=0 && n<i->inpsize ; n++)
   {
      mvaddch(i->y, i->x+n, p[n]);
   }

   return(1);
}


static inp_clear_scr(i)
INP *i;
{
   int n;

   for (n=0 ; n<i->inpsize ; n++)
      mvaddch(i->y, i->x+n, ' ');
}

