#include <stdio.h>
#include <malloc.h>

#include "memdebug.h"


/***********************************************************************
   Memory debug.
***********************************************************************/

#ifdef MEMORYDEBUG

#define USESIZE 4000
static void *memdebug_used[USESIZE];
static unsigned int memdebug_size[USESIZE];
static int memdebug_first=1;

#undef free
#undef malloc
#undef realloc
#undef strdup


memerror(str,size)
char *str;
unsigned int size;
{
   printf("\n\r*** MEMORY ERROR ***\n\r");
   printf("%s [size %u]\n", str, size);

   getchar();
}


void *ttmaloc(size)
unsigned int size;
{
   void *p;
   int n, found=0;

   if (memdebug_first)
   {
      for (n=0 ; n<USESIZE ; n++)
      {
	 memdebug_used[n] = NULL;
	 memdebug_size[n] = 0;
      }
      memdebug_first=0;
   }

   p = malloc(size);
   if (p==NULL)
   {
      memerror("malloc()", size);
      return(NULL);
   }

   for (n=0 ; n<USESIZE && !found ; n++)
      if (memdebug_used[n] == NULL)
	 found=n+1;

   if (!found)
   {
      memerror("Memory debugger full",0);
      return(p);
   }

   found--;
   memdebug_used[found] = p;
   memdebug_size[found] = size;
   return(p);
}


void *ttrealloc(p, size)
void *p;
unsigned int size;
{
   int n, found=0;
   void *newp;

   if (memdebug_first)
   {
      for (n=0 ; n<USESIZE ; n++)
	 memdebug_used[n] = NULL;
      memdebug_first=0;
   }

   newp = realloc(p, size);
   if (newp == NULL)
   {
      memerror("realloc()", size);
      return(NULL);
   }

   if (newp == p)
      return(newp);

   for (n=0 ; n<USESIZE && !found ; n++)
      if (memdebug_used[n] == p)
      {
	 memdebug_used[n] = newp;
	 memdebug_size[n] = size;
	 found=1;
      }

   if (!found)
      memerror("Reallocating unknown pointer", size);

   return(newp);
}


void ttfree(p)
void *p;
{
   int n;
   int found=0;

   if (p==NULL)
      memerror("Freeing NULL pointer", 0);

   for (n=0 ; n<USESIZE && !found ; n++)
      if (memdebug_used[n] == p)
	 found = n+1;

   if (!found)
   {
      memerror("Freeing unknown pointer", 0);
   }
   else
   {
      found--;
      free(p);
      memdebug_used[found] = NULL;
      memdebug_size[found] = 0;
   }
}


char *ttstrdup(str)
char *str;
{
   char *p;

   p = (char *)ttmaloc(strlen(str)+1);
   if (p == NULL)
   {
      memerror("strdup()", strlen(str)+1);
      return(NULL);
   }

   strcpy(p, str);
   return(p);
}


void tttestused()
{
   int n,m;
   int first=1;

   for (n=0 ; n<USESIZE ; n++)
      if (memdebug_used[n] != NULL)
      {
	 if (first)
	    printf("\nMemory debugger allocated memoryblocks :\n");
	 first=0;
	 printf("     %p [", memdebug_used[n]);
	 for (m=0 ; m<20 && m<memdebug_size[n] ; m++)
	    if (((char *)memdebug_used[n])[m] < 32)
	       printf(".");
	    else
	       printf("%c", ((char *)memdebug_used[n])[m]);
	 printf("] : %u bytes\n", memdebug_size[n]);
      }
}

#endif /* MEMORYDEBUG */
